// MatsuuraDoc.cpp : implementation of the CMatsuuraDoc class
//

#include "StdAfx.h"
#include "mainfrm.h"
#include "leftview.h"
#include "Matsuura.h"

#include "MatsuuraDoc.h"
#include "MatsuuraView.h"

#include "AISDialogs.h"
#include "MachineDlg.h"
#include "ToolDlg.h"

#ifdef _DEBUG
//#define new DEBUG_NEW  // by cascade
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

char* GetAppPath()
{
	CString path;
	path=AfxGetApp()->m_pszHelpFilePath;
	path=path.Left(path.ReverseFind ('\\')+1);
	char str[1024];
	strcpy(str, LPCSTR(path));
	return str;
};

/////////////////////////////////////////////////////////////////////////////
// CMatsuuraDoc

IMPLEMENT_DYNCREATE(CMatsuuraDoc, CDocument)

BEGIN_MESSAGE_MAP(CMatsuuraDoc, CDocument)
	//{{AFX_MSG_MAP(CMatsuuraDoc)
	ON_COMMAND(ID_OBJECT_COLOR, OnObjectColor)
	ON_UPDATE_COMMAND_UI(ID_OBJECT_COLOR, OnUpdateObjectColor)
	ON_COMMAND(ID_OBJECT_SHADING, OnObjectShading)
	ON_UPDATE_COMMAND_UI(ID_OBJECT_SHADING, OnUpdateObjectShading)
	ON_COMMAND(ID_OBJECT_WIREFRAME, OnObjectWireframe)
	ON_UPDATE_COMMAND_UI(ID_OBJECT_WIREFRAME, OnUpdateObjectWireframe)
	ON_COMMAND(ID_OBJECT_TRANPARENCY, OnObjectTransparency)
	ON_UPDATE_COMMAND_UI(ID_OBJECT_TRANPARENCY, OnUpdateObjectTransparency)
	ON_COMMAND(ID_FILE_EXPORTIMAGE, OnFileExportimage)
	ON_COMMAND(ID_SIMULATION_OPTIONS, OnSimulationOptions)
	ON_COMMAND(ID_SIMULATION_ITERATION, OnSimulationIteration)
	ON_COMMAND(ID_PATH_ADD_SIMPLE_FACE, OnPathAddSimpleFace)
	ON_COMMAND(ID_PATH_ADD_COMPLEX_FACE, OnPathAddComplexFace)
	ON_COMMAND(ID_SIMULATION_COMPUTESIMULATION, OnSimulationComputesimulation)
	ON_COMMAND(ID_SIMULATION_RUNSIMULATION, OnSimulationRunsimulation)
	ON_COMMAND(ID_SIMULATION_SAVE_CARTESIAN, OnSimulationSaveCartesian)
	ON_COMMAND(ID_SIMULATION_SAVE_INVERSE, OnSimulationSaveInverse)
	ON_COMMAND(ID_MACHINE_CUSTOM, OnMachineCustom)
	ON_COMMAND(ID_PART_OPEN, OnPartOpen)
	ON_COMMAND(ID_TOOL_CUSTOM, OnToolCustom)
	ON_UPDATE_COMMAND_UI(ID_SIMULATION_RUNSIMULATION, OnUpdateSimulationRunsimulation)
	ON_UPDATE_COMMAND_UI(ID_PATH_ADD_SIMPLE_FACE, OnUpdatePathAddSimpleFace)
	ON_UPDATE_COMMAND_UI(ID_PATH_ADD_COMPLEX_FACE, OnUpdatePathAddComplexFace)
	ON_UPDATE_COMMAND_UI(ID_SIMULATION_COMPUTESIMULATION, OnUpdateSimulationComputesimulation)
	ON_COMMAND(ID_MACHINE_MEANARTICULARPOSITIONS, OnMachineMeanarticularpositions)
	ON_COMMAND(ID_CLOSE_CONTEXT, OnCloseContext)
	//}}AFX_MSG_MAP

END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CMatsuuraDoc construction/destruction

CMatsuuraView* CMatsuuraDoc::Get3dView()
{
	return ((CMainFrame*)AfxGetMainWnd())->GetRightPane();
}

CLeftView* CMatsuuraDoc::GetTreeView()
{
	return ((CMainFrame*)AfxGetMainWnd())->GetLeftPane();
}


CMatsuuraDoc::CMatsuuraDoc()
{
	Handle(Graphic3d_WNTGraphicDevice) theGraphicDevice = 
		((CMatsuuraApp*)AfxGetApp())->GetGraphicDevice();
	myViewer = new V3d_Viewer(theGraphicDevice,(short *) "Visu3D");
	myViewer->SetDefaultLights();
	myViewer->SetLightOn();
	myAISContext =new AIS_InteractiveContext(myViewer);

	A_Coord=NULL;
	C_Coord=NULL;
}

CMatsuuraDoc::~CMatsuuraDoc()
{
	FreeCoord();
}

void CMatsuuraDoc::FreeCoord()
{
	if(A_Coord)
	{
		for (int i=0 ; i<nb_Coord ; i++)
			delete A_Coord[i];
		delete A_Coord;
		A_Coord=NULL;
	}

	if(C_Coord)
	{
		for (int i=0 ; i<nb_Coord ; i++)
			delete C_Coord[i];
		delete C_Coord;
		C_Coord=NULL;
	}
	nb_Coord=0;
}

BOOL CMatsuuraDoc::OnNewDocument()
{
	if (!CDocument::OnNewDocument()) 
		return FALSE;

	myAISContext->CloseAllContexts();
	myAISContext->EraseAll();
	Machine.Init();
	Path.Init();
	Path.SetMachine(&Machine);
	opened=false;
	precision=1e-4;
	discret=50;
	max_iter=10000;
	SurfType=SIMPLE_FACE;
	CurMethod=RMR_METHOD;
	CurCriteria=GLOBAL_CRITERIA;
	computed=false;
	FreeCoord();

	BRep_Builder B;
	TopoDS_Shape S1, S2, S3, S4, S5;

	char AppPath[1024];
	strcpy(AppPath, GetAppPath());
	char Name[1024];

	strcpy(Name, AppPath);
	strcat(Name,"\\Machine\\Frame.brep");
	BRepTools::Read(S1,Name,B);
	strcpy(Name, AppPath);
	strcat(Name,"\\Machine\\Table_XY.brep");
	BRepTools::Read(S2,Name,B);
	strcpy(Name, AppPath);
	strcat(Name,"\\Machine\\Table_Z.brep");
	BRepTools::Read(S3,Name,B);
	strcpy(Name, AppPath);
	strcat(Name,"\\Machine\\Table_A.brep");
	BRepTools::Read(S4,Name,B);
	strcpy(Name, AppPath);
	strcat(Name,"\\Machine\\Table_C.brep");
	BRepTools::Read(S5,Name,B);

	if (S1.IsNull() || S2.IsNull() || S3.IsNull() || S4.IsNull() || S5.IsNull())
		AfxMessageBox("Shape(s) not found.\nCheck the Machine directory path!", MB_ICONERROR);
	else
	{
		gp_Trsf T;

		T.SetTranslation(gp_Vec(0,150,50+Machine.Tool->GetLength()));
		Machine.Table_Z.SetContext(myAISContext);
		Machine.Table_Z.SetShape(S3);
		Machine.Table_Z.SetColor(Quantity_NOC_GREEN);
		Machine.Table_Z.SetMaterial(Graphic3d_NOM_PLASTIC);
		Machine.Table_Z.SetTrsf(T);
		Machine.Table_Z.Display();

		T.SetTranslation(gp_Vec(0,0,-36.96));
		Machine.Table_C.SetContext(myAISContext);
		Machine.Table_C.SetShape(S5);
		Machine.Table_C.SetColor(Quantity_NOC_BLUE1);
		Machine.Table_C.SetMaterial(Graphic3d_NOM_PLASTIC);
		Machine.Table_C.SetTrsf(T);
		Machine.Table_C.Display();

		T.SetTranslation(gp_Vec(0,0,-136.96));
		Machine.Table_A.SetContext(myAISContext);
		Machine.Table_A.SetShape(S4);
		Machine.Table_A.SetColor(Quantity_NOC_ORANGE);
		Machine.Table_A.SetMaterial(Graphic3d_NOM_PLASTIC);
		Machine.Table_A.SetTrsf(T);
		Machine.Table_A.Display();

		T.SetTranslation(gp_Vec(0,0,-596.96));
		Machine.Table_XY.SetContext(myAISContext);
		Machine.Table_XY.SetShape(S2);
		Machine.Table_XY.SetColor(Quantity_NOC_YELLOW);
		Machine.Table_XY.SetMaterial(Graphic3d_NOM_PLASTIC);
		Machine.Table_XY.SetTrsf(T);
		Machine.Table_XY.Display();

		Machine.Frame.SetContext(myAISContext);
		Machine.Frame.SetShape(S1);
		Machine.Frame.SetColor(Quantity_NOC_GREEN4);
		Machine.Frame.SetMaterial(Graphic3d_NOM_PLASTIC);
		Machine.Frame.SetTrsf(T);
		Machine.Frame.Display();

		Machine.Tool->CreateShape();
		Machine.Tool->SetContext(myAISContext);
		Machine.Tool->SetColor(Quantity_NOC_WHITE);
		Machine.Tool->SetMaterial(Graphic3d_NOM_PLASTIC);
		Machine.Tool->Display();
	}
	return TRUE;
}

void CMatsuuraDoc::Serialize(CArchive& ar)
{
	
}


void sleep(clock_t wait)
{
   clock_t goal;
   goal = wait + clock();
   while( goal > clock() )
      ;
}

void CMatsuuraDoc::OnFileExportimage()
{
  CFileDialog dlg(FALSE,_T("*.BMP"),NULL,OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT,
                  _T("BMP Files (*.BMP)|*.bmp |GIF Files (*.GIF)|*.gif | XWD Files (*.XWD)|*.xwd||"), 
                  NULL );

  Handle(Aspect_Window) anAspectWindow = Get3dView()->GetView()->Window();
  Handle(WNT_Window) aWNTWindow = Handle(WNT_Window)::DownCast(anAspectWindow);
	bool record=false;
	CString filename;
	if (dlg.DoModal() == IDOK) 
  {
		filename = dlg.GetPathName();
    CString ext = dlg.GetFileExt();
    if (ext == "bmp")     aWNTWindow->SetOutputFormat ( WNT_TOI_BMP );
    if (ext == "gif")     aWNTWindow->SetOutputFormat ( WNT_TOI_GIF );
    if (ext == "xwd")     aWNTWindow->SetOutputFormat ( WNT_TOI_XWD );
		record=true;
  }
	if (record)
		{
		CWaitCursor c;
		sleep(1000);
		aWNTWindow->Dump ((Standard_CString)(LPCTSTR)filename);
		}
}

void CMatsuuraDoc::OnMachineCustom() 
{
	CMachineDlg dlg;
	dlg.m_Amax=Machine.Amax;
	dlg.m_Amin=Machine.Amin;
	dlg.m_Cmax=Machine.Cmax;
	dlg.m_Cmin=Machine.Cmin;
	dlg.m_Xmax=Machine.Xmax;
	dlg.m_Xmin=Machine.Xmin;
	dlg.m_Ymax=Machine.Ymax;
	dlg.m_Ymin=Machine.Ymin;
	dlg.m_Zmax=Machine.Zmax;
	dlg.m_Zmin=Machine.Zmin;

	if (dlg.DoModal()==IDOK)
	{
		Machine.Amax=dlg.m_Amax;
		Machine.Amin=dlg.m_Amin;
		Machine.Cmax=dlg.m_Cmax;
		Machine.Cmin=dlg.m_Cmin;
		Machine.Xmax=dlg.m_Xmax;
		Machine.Xmin=dlg.m_Xmin;
		Machine.Ymax=dlg.m_Ymax;
		Machine.Ymin=dlg.m_Ymin;
		Machine.Zmax=dlg.m_Zmax;
		Machine.Zmin=dlg.m_Zmin;
	}
}

void CMatsuuraDoc::OnMachineMeanarticularpositions() 
{
	CString text;
	text.Format("Cmoy[deg]=%lg\nAmoy[deg]=%lg\nXmoy[mm]=%lg\nYmoy[mm]=%lg\nZmoy[mm]=%lg",
		Machine.GetCmoy(),
		Machine.GetAmoy(),
		Machine.GetXmoy(),
		Machine.GetYmoy(),
		Machine.GetZmoy());
	AfxMessageBox(text);
}

void CMatsuuraDoc::OnToolCustom() 
{
	CToolDlg dlg;
	dlg.Set(Machine.Tool->GetDiameter(), Machine.Tool->GetLength());

	if (dlg.DoModal()==IDOK)
	{
		//on deplace la broche pour que l'outil soit toujours a h=0
		gp_Trsf T1, T2;
		T1=Machine.Table_Z.GetTrsf();
		T2.SetTranslation(gp_Vec(0,0,dlg.m_l-Machine.Tool->GetLength()));
		Machine.Table_Z.SetTrsf(T1*T2);
		computed=false;
		Machine.Init();
		Part.Init();
		Machine.Tool->Erase();
		Machine.Tool->SetDiameter(dlg.m_d);
		Machine.Tool->SetLength(dlg.m_l);
		Machine.Tool->CreateShape();
		Machine.Tool->SetColor(Quantity_NOC_WHITE);
		Machine.Tool->SetMaterial(Graphic3d_NOM_PLASTIC);
		Machine.Tool->Display();
		FreeCoord();
	}	
}

void CMatsuuraDoc::OnPartOpen() 
{
	static bool first=true;

	CFileDialog dlg(TRUE,
					NULL,
					NULL,
					OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT,
					"BREP Files (*.brep , *.BREP)|*.brep; *.BREP|rle Files (*.rle, *.RLE)|*.rle; *.RLE|All Files (*.*)|*.*||", 
					NULL );

	if (dlg.DoModal()==IDOK)
	{
		myAISContext->CloseAllContexts();
		if (!first)
		{
			OnNewDocument();
			FreeCoord();
			computed=false;
			Part.Erase();
			Part.Init();
			Machine.Init();
			Path.Init();
			UpdateAllViews(((CMainFrame*)AfxGetMainWnd())->GetRightPane());
		}
		first=false;
		opened=true;

		BRep_Builder B;
		TopoDS_Shape S;
		BRepTools::Read(S,(Standard_CString)(LPCSTR)dlg.GetPathName(),B);
		Part.SetContext(myAISContext);
		Part.SetShape(S);
		Part.SetColor(Quantity_NOC_RED);
		Part.SetMaterial(Graphic3d_NOM_PLASTIC);
		Part.Display();
	}
}

void CMatsuuraDoc::OnSimulationOptions() 
{
	COptions dlg(CurMethod);

	dlg.m_Precision = precision;
	dlg.m_Discret = discret;
	dlg.m_MaxIter = max_iter;
	dlg.m_MAD = Machine.MAD;
	dlg.m_WFo = Machine.WFo*100;
	dlg.m_Delay = Get3dView()->delay;
	dlg.Criteria = CurCriteria;

	if (dlg.DoModal()==IDOK)
	{
		precision=dlg.m_Precision;
		discret=dlg.m_Discret;
		max_iter=dlg.m_MaxIter;
		Get3dView()->delay=dlg.m_Delay;
		Machine.MAD=dlg.m_MAD;
		Machine.WFo=dlg.m_WFo/100;
		CurMethod=dlg.Method;
		CurCriteria=dlg.Criteria;
	}
}

void CMatsuuraDoc::OnSimulationIteration() 
{
	CString text;
	double min=1e10;
	double max=0;

	text.Format("Minimum number of iterations : %d\nMaximum number of iterations : %d", Path.GetMinNbIteration(), Path.GetMaxNbIteration());
	AfxMessageBox(text);
}

/////////////////////////////////////////////////////////////////////////////
// CMatsuuraDoc diagnostics

#ifdef _DEBUG
void CMatsuuraDoc::AssertValid() const
{
	CDocument::AssertValid();
}

void CMatsuuraDoc::Dump(CDumpContext& dc) const
{
	CDocument::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CMatsuuraDoc 3d viewer

void CMatsuuraDoc::OnObjectColor() 
{
	myAISContext->InitCurrent();
    Quantity_Color CSFColor = myAISContext->Color(myAISContext->Current());
    COLORREF MSColor = RGB (CSFColor.Red()*255.,CSFColor.Green()*255.,CSFColor.Blue()*255.);
	
	CColorDialog dlgColor(MSColor);
	if (dlgColor.DoModal() == IDOK)
	{
	  MSColor = dlgColor.GetColor();
	  CSFColor = Quantity_Color (GetRValue(MSColor)/255.,GetGValue(MSColor)/255.,
						         GetBValue(MSColor)/255.,Quantity_TOC_RGB); 
	  for (;myAISContext->MoreCurrent ();myAISContext->NextCurrent ())
		  myAISContext->SetColor (myAISContext->Current(),CSFColor.Name());
	}
}
void CMatsuuraDoc::OnUpdateObjectColor(CCmdUI* pCmdUI) 
{
    myAISContext->InitCurrent();
	pCmdUI->Enable (myAISContext->MoreCurrent ());	
}

void CMatsuuraDoc::OnObjectWireframe() 
{
  for(myAISContext->InitCurrent();myAISContext->MoreCurrent();myAISContext->NextCurrent())
        myAISContext->SetDisplayMode(myAISContext->Current(),0);
}
void CMatsuuraDoc::OnUpdateObjectWireframe(CCmdUI* pCmdUI) 
{
  bool OneOrMoreInShading = Standard_False;
  for (myAISContext->InitCurrent();myAISContext->MoreCurrent ();myAISContext->NextCurrent ())
    if (myAISContext->IsDisplayed(myAISContext->Current(),1)) OneOrMoreInShading=Standard_True;
	pCmdUI->Enable (OneOrMoreInShading);	
}

void CMatsuuraDoc::OnObjectShading() 
{
  CWaitCursor aWaitCursor;
  for(myAISContext->InitCurrent();myAISContext->MoreCurrent();myAISContext->NextCurrent())
      myAISContext->SetDisplayMode(myAISContext->Current(),1);
}

void CMatsuuraDoc::OnUpdateObjectShading(CCmdUI* pCmdUI) 
{
  bool OneOrMoreInWireframe = Standard_False;
  for (myAISContext->InitCurrent();myAISContext->MoreCurrent ();myAISContext->NextCurrent ())
    if (myAISContext->IsDisplayed(myAISContext->Current(),0)) OneOrMoreInWireframe=Standard_True;
	pCmdUI->Enable (OneOrMoreInWireframe);	
}

void CMatsuuraDoc::OnObjectTransparency()
{
	CDialogTransparency DialBox(myAISContext);
	DialBox.DoModal();	
}

void CMatsuuraDoc::OnUpdateObjectTransparency(CCmdUI* pCmdUI) 
{
  bool OneOrMoreInShading = Standard_False;
  for (myAISContext->InitCurrent();myAISContext->MoreCurrent ();myAISContext->NextCurrent ())
    if (myAISContext->IsDisplayed(myAISContext->Current(),1)) OneOrMoreInShading=Standard_True;
	pCmdUI->Enable (OneOrMoreInShading);	
}

/////////////////////////////////////////////////////////////////////////////
// CMatsuuraDoc robotique

void CMatsuuraDoc::DrawStep(double X, double Y, double Z, double A, double C)
{
	double Ca, Sa, Cc, Sc;
	double l=Machine.Tool->GetLength();
	double d=Machine.d;

	Ca=cos(A*PI/180);
	Sa=sin(A*PI/180);
	Cc=cos(C*PI/180);
	Sc=sin(C*PI/180);

	
	Machine.Tool->Transform(	1,	0,	0,	0,
					0,	1,	0,	0,
					0,	0,	1,	Z-l);
	Machine.Table_Z.Transform(	1,	0,	0,	0,
						0,	1,	0,	0,
						0,	0,	1,	Z-l);

	Part.Transform(	Cc,		-Sc,	0,		X,
					Ca*Sc,	Ca*Cc,	-Sa,	-Sa*d+Y,
					Sa*Sc,	Sa*Cc,	Ca,		Ca*d-d);

	Machine.Table_C.Transform(	Cc,		-Sc,	0,		X,
					Ca*Sc,	Ca*Cc,	-Sa,	-Sa*d+Y,
					Sa*Sc,	Sa*Cc,	Ca,		Ca*d-d);

	Machine.Table_A.Transform(	1,	0,	0,		X,
						0,	Ca,	-Sa,	-Sa*d+Y,
						0,	Sa,	Ca,		Ca*d-d);
	Machine.Table_XY.Transform(	1,	0,	0,	X,
						0,	1,	0,	Y,
						0,	0,	1,	0);

	myAISContext->UpdateCurrentViewer();
}


/////////////////////////////////////////////////////////////////////////////
// CMatsuuraDoc interface

bool CMatsuuraDoc::OnStep() 
{
	if (!computed)
	{
		AfxMessageBox("Nothing computed !");
		Get3dView()->OnStop();
		return false;
	}

	double X, Y, Z, A, C;
	
	myAISContext->CloseAllContexts();

	if (NumCurPos > nb_Coord-1)
		NumCurPos=0;
	else if (NumCurPos < 0)
		NumCurPos=nb_Coord-1;

	X=A_Coord[NumCurPos][0];
	Y=A_Coord[NumCurPos][1];
	Z=A_Coord[NumCurPos][2];
	A=A_Coord[NumCurPos][3];
	C=A_Coord[NumCurPos][4];
	
	DrawStep(X, Y, Z, A, C);
	return true;
}

void CMatsuuraDoc::OnPathAddSimpleFace() 
{
	if (computed)
	{
		NumCurPos=-1;
		OnStep();
	}
	myAISContext->CloseAllContexts();
	myAISContext->OpenLocalContext();
	myAISContext->ActivateStandardMode(TopAbs_FACE);
	SurfType=SIMPLE_FACE;
	Path.ArrayOfEdge.Clear();
	UpdateAllViews(((CMainFrame*)AfxGetMainWnd())->GetRightPane());
}

void CMatsuuraDoc::OnPathAddComplexFace() 
{
	if (computed)
	{
		NumCurPos=-1;
		OnStep();
	}
	myAISContext->CloseAllContexts();
	myAISContext->OpenLocalContext();
	myAISContext->ActivateStandardMode(TopAbs_FACE);
	SurfType=COMPLEX_FACE;
	Path.ArrayOfEdge.Clear();
	UpdateAllViews(((CMainFrame*)AfxGetMainWnd())->GetRightPane());
}

void CMatsuuraDoc::AddComplexFace(TopoDS_Face &F)
{
	Path.SetFace(F);
	UpdateAllViews(((CMainFrame*)AfxGetMainWnd())->GetRightPane());
}

void CMatsuuraDoc::AddComplexEdge(TopoDS_Edge &E)
{
	Path.AddEdge(E);
	UpdateAllViews(((CMainFrame*)AfxGetMainWnd())->GetRightPane());
}

void CMatsuuraDoc::AddSimpleFace(TopoDS_Face &F)
{
	Path.SetFace(F);
	Path.ComputeEdge();
	UpdateAllViews(((CMainFrame*)AfxGetMainWnd())->GetRightPane());
}

void CMatsuuraDoc::OnSimulationComputesimulation() 
{
	int i,k;

	NumCurPos=0;

	FreeCoord();

	Path.SetDiscretisation(discret);
	Path.SetMaxIter(max_iter);
	Path.SetPrecision(precision);
	Path.InitPath();
	Path.SetMethod(CurMethod);
	Path.SetCriteria(CurCriteria);
	Path.ComputePointAndNormal();
	nb_Coord=Path.GetNbPoint();
	Path.ComputePath();

	A_Coord = new double*[nb_Coord];
	C_Coord = new double*[nb_Coord];

	for (i=0 ; i<nb_Coord ; i++)
		{
			A_Coord[i] = new double[NB_JOINT];
			C_Coord[i] = new double[6];
			for (k=0 ; k<NB_JOINT ; k++)
				A_Coord[i][k]=Path.A_Coord[i][k];
			C_Coord[i][0]=Path.ArrayOfPoint[i].X();
			C_Coord[i][1]=Path.ArrayOfPoint[i].Y();
			C_Coord[i][2]=Path.ArrayOfPoint[i].Z();
			C_Coord[i][3]=Path.ArrayOfNormal[i].X();
			C_Coord[i][4]=Path.ArrayOfNormal[i].Y();
			C_Coord[i][5]=Path.ArrayOfNormal[i].Z();
		}
	computed=true;
}

void CMatsuuraDoc::OnSimulationRunsimulation() 
{
	CFileDialog dlg(true);
	if (dlg.DoModal()==IDOK)
	{
		ReadA_File(dlg.GetPathName());
		computed=true;
		CMainFrame* pFrame=dynamic_cast<CMainFrame*>(AfxGetMainWnd());
		pFrame->SetActiveView(pFrame->GetRightPane());
		NumCurPos=-1;
		OnStep();
	}
}

bool CMatsuuraDoc::ReadA_File(CString &text)
{
	fstream f;
	int i, j;

	f.clear();
	f.open(LPCSTR(text), ios::in);
	if (f.fail ())
		return false;
	
	FreeCoord();

	f >> nb_Coord;

	A_Coord = new double*[nb_Coord];

	for (i=0 ; i<nb_Coord ; i++)
	{
		A_Coord[i] = new double[NB_JOINT];
		for (j=0 ; j<NB_JOINT ; j++)
			f >> A_Coord[i][j];
	}
	f.close();
	return true;
}


bool CMatsuuraDoc::OnSimulationSaveCartesian() 
{
	ofstream f;
	int i;
	CString text;
	CFileDialog dlg(false);

	if (dlg.DoModal()==IDOK)
	{
		f.clear();
		f.open(LPCSTR(dlg.GetPathName()));
		if (f.fail ())
			return false;

		Path.SetDiscretisation(discret);
		Path.InitPath();
		Path.SetMethod(CurMethod);
		Path.ComputePointAndNormal();
		for (i=0 ; i<Path.GetNbPoint() ; i++)
		{
			f<<Path.ArrayOfPoint[i].X()<<"\t";
			f<<Path.ArrayOfPoint[i].Y()<<"\t";
			f<<Path.ArrayOfPoint[i].Z()<<"\t";
			f<<Path.ArrayOfNormal[i].X()<<"\t";
			f<<Path.ArrayOfNormal[i].Y()<<"\t";
			f<<Path.ArrayOfNormal[i].Z()<<"\t";
			f<<endl;
		}
		
		f.close();
		return true;
	}
	return false;
}
bool CMatsuuraDoc::OnSimulationSaveInverse() 
{
	ofstream f;
	int i;
	CString text;
	CFileDialog dlg(false);

	if (computed)
	{
		if (dlg.DoModal()==IDOK)
		{
			f.clear();
			f.open(LPCSTR(dlg.GetPathName()));
			if (f.fail ())
				return false;
	
			f << nb_Coord <<endl;

			for (i=0 ; i<nb_Coord ; i++)
				f<<A_Coord[i][0]<<"\t"<<A_Coord[i][1]<<"\t"<<A_Coord[i][2]<<"\t"<<A_Coord[i][3]<<"\t"<<A_Coord[i][4]<< endl;
			f.close();
			return true;
		}
	}
	else
		AfxMessageBox("Compute simulation first !", MB_ICONSTOP);

	return false;
}

void CMatsuuraDoc::ReachPoint(gp_Pnt &p, gp_Dir &o) 
{
	double X, Y, Z, A, C;

	Path.MGI(p, o, X, Y, Z, A, C);
	DrawStep(X, Y, Z, A, C);
}

void CMatsuuraDoc::OnUpdateSimulationRunsimulation(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(opened);	
}

void CMatsuuraDoc::OnUpdatePathAddSimpleFace(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(opened);	
}

void CMatsuuraDoc::OnUpdatePathAddComplexFace(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(opened);	
}

void CMatsuuraDoc::OnUpdateSimulationComputesimulation(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(Path.GetNbEdge());	
}

void CMatsuuraDoc::OnCloseContext() 
{
	myAISContext->CloseAllContexts();
}
