// MatsuuraDoc.h : interface of the CMatsuuraDoc class
//
/////////////////////////////////////////////////////////////////////////////

#if !defined(AFX_MatsuuraDOC_H__20EAA2EE_4788_41E8_92D5_BFB89F557366__INCLUDED_)
#define AFX_MatsuuraDOC_H__20EAA2EE_4788_41E8_92D5_BFB89F557366__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#include "MatsuuraObject.h"
#include "MatsuuraTool.h"
#include "MatsuuraPath.h"
#include "MatsuuraMachine.h"
#include "options.h"


//enumeration
enum SurfType_
{
	SIMPLE_FACE,
	COMPLEX_FACE
};

class CMatsuuraView;
class CLeftView;
class CMainFrame;

class CMatsuuraDoc : public CDocument
{
protected: // create from serialization only
	CMatsuuraDoc();
	DECLARE_DYNCREATE(CMatsuuraDoc)


// Attributes
public:
	SurfType_ SurfType;

	int nb_Coord;		//nombres de coordonnees 
	int NumCurPos;		//position courante
	double precision;	//precision de calcul du RMR
	int discret;		//nombre de positions sur une courbe
	int max_iter;		//nombre maxi d'iterations pour la boucle de RMR
	bool computed;
	bool opened;

	MatsuuraMachine Machine;
	MatsuuraObject Part;
	MatsuuraPath Path;

	CurMethod_ CurMethod;
	CurCriteria_ CurCriteria;
	double** A_Coord;	//coordonnees articulaires
	double** C_Coord;	//coordonnees articulaires
	

// Operations
public:
	CMatsuuraView* Get3dView();
	CLeftView* GetTreeView();
	void FreeCoord();
	void AddCycle();
	void AddComplexFace(TopoDS_Face &F);
	void AddComplexEdge(TopoDS_Edge &E);
	void AddSimpleFace(TopoDS_Face &F);

	bool ReadA_File(CString &text);

	void DrawStep(double X, double Y, double Z, double A, double C);
	void ReachPoint(gp_Pnt &p, gp_Dir &o);
	bool OnStep();

// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CMatsuuraDoc)
	public:
	virtual BOOL OnNewDocument();
	virtual void Serialize(CArchive& ar);
	//}}AFX_VIRTUAL

// Implementation
public:
	virtual ~CMatsuuraDoc();
#ifdef _DEBUG
	virtual void AssertValid() const;
	virtual void Dump(CDumpContext& dc) const;
#endif

protected:

// Generated message map functions
public:
	//{{AFX_MSG(CMatsuuraDoc)
	afx_msg void OnObjectColor();
	afx_msg void OnUpdateObjectColor(CCmdUI* pCmdUI);
	afx_msg void OnObjectShading();
	afx_msg void OnUpdateObjectShading(CCmdUI* pCmdUI);
	afx_msg void OnObjectWireframe();
	afx_msg void OnUpdateObjectWireframe(CCmdUI* pCmdUI);
    afx_msg void OnObjectTransparency();
	afx_msg void OnUpdateObjectTransparency(CCmdUI* pCmdUI);
	afx_msg void OnFileExportimage();
	afx_msg void OnSimulationOptions();
	afx_msg void OnSimulationIteration();
	afx_msg void OnPathAddSimpleFace();
	afx_msg void OnPathAddComplexFace();
	afx_msg void OnSimulationComputesimulation();
	afx_msg void OnSimulationRunsimulation();
	afx_msg bool OnSimulationSaveCartesian();
	afx_msg bool OnSimulationSaveInverse();
	afx_msg void OnMachineCustom();
	afx_msg void OnPartOpen();
	afx_msg void OnToolCustom();
	afx_msg void OnUpdateSimulationRunsimulation(CCmdUI* pCmdUI);
	afx_msg void OnUpdatePathAddSimpleFace(CCmdUI* pCmdUI);
	afx_msg void OnUpdatePathAddComplexFace(CCmdUI* pCmdUI);
	afx_msg void OnUpdateSimulationComputesimulation(CCmdUI* pCmdUI);
	afx_msg void OnMachineMeanarticularpositions();
	afx_msg void OnCloseContext();
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()

private:
	Handle_V3d_Viewer myViewer;
	Handle_AIS_InteractiveContext myAISContext;

public :
  	Handle_AIS_InteractiveContext& GetAISContext(){ return myAISContext; };
	Handle_V3d_Viewer GetViewer()  { return myViewer; };

};

/////////////////////////////////////////////////////////////////////////////

//{{AFX_INSERT_LOCATION}}
// Microsoft Visual C++ will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_MatsuuraDOC_H__20EAA2EE_4788_41E8_92D5_BFB89F557366__INCLUDED_)
