// MatsuuraPath.cpp: implementation of the MatsuuraPath class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "matsuura.h"
#include "MatsuuraPath.h"
#include "MatsuuraObject.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

#include <math.h>

//fonctions globales
gp_XYZ Vect(gp_Mat &M)
{
	gp_XYZ V;
	
	V.SetX(0.5*(M(3,2)-M(2,3)));
	V.SetY(0.5*(M(1,3)-M(3,1)));
	V.SetZ(0.5*(M(2,1)-M(1,2)));

	return V;
}

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

MatsuuraPath::MatsuuraPath()
{
	discret=50;	//discretisation de la trajectoire
	max_iter=10000;
	precision=1e-4;
	min_nb_iter=1e10;
	max_nb_iter=0;
	StartPnt.SetCoord(0,0,200);
	EndPnt.SetCoord(0,0,200);
	StartDir.SetCoord(0,0,1);
	EndDir.SetCoord(0,0,1);
	A_Coord=NULL;
	CurMethod=RMR_METHOD;
	CurCriteria=GLOBAL_CRITERIA;
}

MatsuuraPath::~MatsuuraPath()
{
	FreeA_Coord();
}

void MatsuuraPath::ResetArray()
{
	ArrayOfEdge.Clear();
	ArrayOfCurve.RemoveAll();
	ArrayOfCurve2d.RemoveAll();
	ArrayOfPoint.RemoveAll();
	ArrayOfNormal.RemoveAll();
}

void MatsuuraPath::Init()
{
	ArrayOfEdge.Clear();
	ArrayOfCurve.RemoveAll();
	ArrayOfCurve2d.RemoveAll();
	ArrayOfPoint.RemoveAll();
	ArrayOfNormal.RemoveAll();
	FreeA_Coord();
	min_nb_iter=1e10;
	max_nb_iter=0;
	StartPnt.SetCoord(0,0,200);
	EndPnt.SetCoord(0,0,200);
	StartDir.SetCoord(0,0,1);
	EndDir.SetCoord(0,0,1);
}

void MatsuuraPath::InitPath()
{
	ArrayOfPoint.RemoveAll();
	ArrayOfNormal.RemoveAll();
	ArrayOfMat.RemoveAll();
	FreeA_Coord();
}

void MatsuuraPath::FreeA_Coord()
{
	if(A_Coord)
	{
		for (int i=0 ; i<ArrayOfPoint.GetSize() ; i++)
			delete A_Coord[i];
		delete A_Coord;
		A_Coord=NULL;
	}
}

void MatsuuraPath::SetEdge(int index, TopoDS_Edge &E)
{
	if (index>=1)
		ArrayOfEdge.SetValue(index,E);
}

TopoDS_Edge MatsuuraPath::GetEdge(int index)
{
	if (index>=1 && index<=ArrayOfEdge.Length())
		return TopoDS::Edge(ArrayOfEdge.Value(index));
}
	
void MatsuuraPath::SetCurve(int index, BRepAdaptor_Curve &C)
{
	if (index>=0)
		ArrayOfCurve.SetAtGrow(index,C);
}

BRepAdaptor_Curve& MatsuuraPath::GetCurve(int index)
{
	if (index>=0 && index<ArrayOfCurve.GetSize())
		return ArrayOfCurve[index];
}
	
void MatsuuraPath::SetCurve2d(int index, BRepAdaptor_Curve2d &C2d)
{
	if (index>=0)
		ArrayOfCurve2d.SetAtGrow(index,C2d);
}

BRepAdaptor_Curve2d& MatsuuraPath::GetCurve2d(int index)
{
	if (index>=0 && index<ArrayOfCurve2d.GetSize())
		return ArrayOfCurve2d[index];
}

void MatsuuraPath::SetPoint(int index, gp_Pnt &P)
{
	if (index>=0)
		ArrayOfPoint.SetAtGrow(index,P);
}


gp_Pnt& MatsuuraPath::GetPoint(int index)
{
	if (index>=0 && index<ArrayOfPoint.GetSize())
		return ArrayOfPoint[index];
}

void MatsuuraPath::SetNormal(int index, gp_Dir &N)
{
	if (index>=0)
		ArrayOfNormal.SetAtGrow(index,N);
}

gp_Dir& MatsuuraPath::GetNormal(int index)
{
	if (index>=0 && index<ArrayOfNormal.GetSize())
		return ArrayOfNormal[index];
}

void MatsuuraPath::ComputeEdge()
{
	TopExp_Explorer ExWire, ExEdge;
	TopoDS_Edge E;

	ArrayOfEdge.Clear();
	
	for (ExWire.Init(F,TopAbs_WIRE); ExWire.More(); ExWire.Next()) 
		for (ExEdge.Init(ExWire.Current(),TopAbs_EDGE); ExEdge.More(); ExEdge.Next()) 
		{
			E=TopoDS::Edge(ExEdge.Current());
			ArrayOfEdge.Append(E);
		}
}

void MatsuuraPath::ComputePointAndNormal()
{
	int i,j,length;
	double u, delta;
	gp_XYZ P1, P2, P3, P4;
	CArray<int, int> Orientation;
	gp_Pnt P;
	gp_Dir N;
	gp_Pnt2d P2d;

	FAdaptor.Initialize(F);

	length=ArrayOfEdge.Length();
	ArrayOfCurve.SetSize(length);
	ArrayOfCurve2d.SetSize(length);
	Orientation.SetSize(length);

	for (i=0 ; i<length ; i++)
	{
		ArrayOfCurve[i].Initialize(TopoDS::Edge(ArrayOfEdge.Value(i+1)));
		ArrayOfCurve2d[i].Initialize(TopoDS::Edge(ArrayOfEdge.Value(i+1)), F);
	}

	//changement d'orientation eventuel des courbes
	//Attention : pas de bon resultat avec TopoDS_Edge::Reverse
	for (i=0 ; i<length ; i++)
		Orientation[i]=1;
	for (i=0 ; i<length-1 ; i++)
	{
		gp_XYZ temp;

		P1=ArrayOfCurve[i].Value(ArrayOfCurve[i].FirstParameter()).Coord();
		P2=ArrayOfCurve[i].Value(ArrayOfCurve[i].LastParameter()).Coord();
		if (Orientation[i]==-1)
		{
			temp=P1;
			P1=P2;
			P2=temp;
		}

		P3=ArrayOfCurve[i+1].Value(ArrayOfCurve[i+1].FirstParameter()).Coord();
		P4=ArrayOfCurve[i+1].Value(ArrayOfCurve[i+1].LastParameter()).Coord();
			
		if ((P4-P2).Modulus()<precision)
			Orientation[i+1]=-1;
		if ((P3-P1).Modulus()<precision)
			Orientation[i]=-1;
		if ((P4-P1).Modulus()<precision)
		{
			Orientation[i]=-1;
			Orientation[i+1]=-1;
		}
	}
	
	//debut du cycle
	if (CurMethod!=RMROPT_METHOD)
	{
		ComputePointAndNormal(0,(Orientation[0]==1 ? ArrayOfCurve[0].FirstParameter() : ArrayOfCurve[0].LastParameter()),P,N);
		StartEndCycle(StartPnt, StartDir, P, N);
	}

	//cycle
	for (i=0 ; i<length ; i++)
	{
		delta = Orientation[i]*(ArrayOfCurve[i].LastParameter()-ArrayOfCurve[i].FirstParameter())/(discret-1);
		u = (Orientation[i]==1 ? ArrayOfCurve[i].FirstParameter() : ArrayOfCurve[i].LastParameter());
		for (j=0; j<discret; j++, u+=delta)
		{
			gp_Mat R;
			ComputePointAndNormal(i,u,P,N);
			ArrayOfPoint.Add(P);
			ArrayOfNormal.Add(N);
			if (CurMethod==RMR_METHOD)
			{
				FillOrientation(N, R, 0, 0);
				ArrayOfMat.Add(R);
			}
		}
	}

	//fin du cycle
	if (CurMethod!=RMROPT_METHOD)
		StartEndCycle(ArrayOfPoint[ArrayOfPoint.GetSize()-1], ArrayOfNormal[ArrayOfNormal.GetSize()-1], EndPnt, EndDir);
	else
	{
		//probleme avec CArray::InsertAt donc...
		CArray<gp_Pnt, gp_Pnt&> StartArrayOfPoint;
		CArray<gp_Pnt, gp_Pnt&> SaveArrayOfPoint;
		CArray<gp_Dir, gp_Dir&> StartArrayOfNormal;
		CArray<gp_Dir, gp_Dir&> SaveArrayOfNormal;
		SaveArrayOfPoint.Copy(ArrayOfPoint);
		SaveArrayOfNormal.Copy(ArrayOfNormal);

		P=ArrayOfPoint[0];
		N=ArrayOfNormal[0];
		P.SetZ(P.Z()+12);
		for (i=0 ; i<5 ; i++)
		{
			P.SetZ(P.Z()-2);
			StartArrayOfPoint.Add(P);
			StartArrayOfNormal.Add(N);
		}
		ArrayOfPoint.RemoveAll();
		ArrayOfPoint.Copy(StartArrayOfPoint);
		ArrayOfPoint.Append(SaveArrayOfPoint);
		ArrayOfNormal.RemoveAll();
		ArrayOfNormal.Copy(StartArrayOfNormal);
		ArrayOfNormal.Append(SaveArrayOfNormal);
	}
}

void MatsuuraPath::ComputePointAndNormal(int i, double u, gp_Pnt &P, gp_Dir &N)
{
	gp_Pnt2d P2d;
	BRepLProp_SLProps SLProps(FAdaptor,1,1e-10);

	P=ArrayOfCurve[i].Value(u);
	ArrayOfCurve2d[i].D0(u,P2d);
	SLProps.SetParameters(P2d.X(), P2d.Y());
	N=SLProps.Normal();
	//on s'assure qu'on usine pas en dessous de la table de la machine !
	if (N.Z()<0)
		N.Reverse();
}

bool MatsuuraPath::FindAngle(gp_Dir &o, double &A, double &C, double oldA, double oldC)
{
	double a,c,Cc,Sc;

	a=acos(o.Z());
	if (a==0 || a/180.0==int(a/180.0))
		c=oldC*PI/180;
	else
	{
		if (ABS(a*180/PI)>15)
		{
			a=ABS(a);
			Sc=o.X()/sin(a);
			Cc=o.Y()/sin(a);
			if (ABS(Cc)<=1 && ABS(Sc)<=1)
				c=atan2(Sc, Cc);
			else
				return false;
		}
		else
		{
			a=ABS(a);
			Sc=o.X()/sin(a);
			Cc=o.Y()/sin(a);

			if (ABS(Cc)<=1 && ABS(Sc)<=1)
				c=atan2(Sc, Cc);
			else
			{
				a=-ABS(a);
				Sc=o.X()/sin(a);
				Cc=o.Y()/sin(a);

				if (ABS(Cc)<=1 && ABS(Sc)<=1)
					c=atan2(Sc, Cc);
				else
					return false;
			}
		}
	}
	C=c*180/PI;
	A=a*180/PI;
	return true;
}

void MatsuuraPath::FillOrientation(double A, double C, gp_Mat &R)
{
	double Ca, Sa, Cc, Sc;

	Ca=cos(A*PI/180);
	Sa=sin(A*PI/180);
	Cc=cos(C*PI/180);
	Sc=sin(C*PI/180);

	R(1,1)=Cc;
	R(2,1)=-Sc;
	R(3,1)=0;
	R(1,2)=Ca*Sc;
	R(2,2)=Ca*Cc;
	R(3,2)=-Sa;
	R(1,3)=Sa*Sc;
	R(2,3)=Sa*Cc;
	R(3,3)=Ca;
}

void MatsuuraPath::FillOrientation(gp_Dir &o, gp_Mat &R, double oldA, double oldC)
{
	double A, C;
	FindAngle(o, A, C, oldA, oldC);
	FillOrientation(A, C, R);
}

void MatsuuraPath::StartEndCycle(gp_Pnt &pi, gp_Dir &oi, gp_Pnt &pf, gp_Dir &of)
{
	gp_XYZ pos=(pf.ChangeCoord()-pi.ChangeCoord())/discret;
	gp_Dir dir;
	gp_Mat R, Rot, Ri, Rf, Q;
	gp_XYZ e;
	gp_Pnt p;
	double sin_angle, cos_angle, angle;
	bool b=oi.IsEqual(of, 1e-6);
	
	FillOrientation(oi, Ri);
	R=Ri;
	if (b)
		dir=oi;
	else
	{
		FillOrientation(of, Rf);
		Q=Ri;
		Q.Transpose();	
		Q=Q*Rf;
		sin_angle=Vect(Q).Modulus();
		cos_angle=(Q(1,1)+Q(2,2)+Q(3,3)-1)/2;
		angle=atan2(sin_angle, cos_angle);
		e=Vect(Q)/sin_angle;
	}
	p=pi.ChangeCoord()-pos;
	for (int i=0 ; i<=discret ; i++)
	{
		p=p.Coord()+pos;
		ArrayOfPoint.Add(p);
		if (!b)
		{
			Rot.SetRotation(e, i*angle / discret);
			R=Ri*Rot;
			dir=R.Column(3);
		}
		ArrayOfNormal.Add(dir);
		if (CurMethod==RMR_METHOD)
			ArrayOfMat.Add(R);
	}
}

void MatsuuraPath::RMR()
{
	CWaitCursor c;
	int i,nb_iter;
	int length=ArrayOfPoint.GetSize();
	double X, Y, Z, A, C, Ca, Sa, Cc, Sc;
	gp_XYZ pi, pf;
	gp_Mat Ri, Rf, Q;
	gp_XYZ delta_p, delta_o;
	math_Vector theta(1,5,0);
	math_Vector deltatheta(1,5,0);
	math_Matrix J(1,6,1,5,0);	//J matrice 6*5 initialisee a 0
	math_Matrix inverseJ(1,5,1,6,0);
	math_Vector v(1,6,0);

	A_Coord = new double*[length];

	for (i=0 ; i<length ; i++)
	{
		nb_iter=0;
		pf=ArrayOfPoint[i].Coord();
		Rf=ArrayOfMat[i];
		do
		{
			nb_iter++;
			//calcul de pi
			X=theta(3);
			Y=theta(4);
			Z=theta(5);
			A=theta(2);
			C=theta(1);
		
			MGD(X, Y, Z, A, C, pi, Ri);
			delta_p=pf-pi;
//			oi=R.Column(3);	//evaluation de l'erreur d'orientation basique
//			delta_o=oi^of;	//et necessitant plus d'iterations
			Q=Ri;
			Q.Transpose();	
			Q=Q*Rf;
			//Q=Ri.Transposed()*Rf; //bug OpenCascade dans transposed declare sur le site
			delta_o=Ri*Vect(Q);
			
			Ca=cos(A*PI/180);
			Sa=sin(A*PI/180);
			Cc=cos(C*PI/180);
			Sc=sin(C*PI/180);

			//determination de J
			J(3,1)=-1;
			J(1,2)=-Cc;
			J(2,2)=Sc;
			J(4,3)=-Cc;
			J(5,3)=Sc;
			J(4,4)=-Ca*Sc;
			J(5,4)=-Ca*Cc;
			J(6,4)=Sa;
			J(4,5)=Sa*Sc;
			J(5,5)=Sa*Cc;
			J(6,5)=Ca;
			J(4,1)=Sc*X-Cc*Ca*Y+Cc*Sa*(Z+d-l);
			J(5,1)=Cc*X+Sc*Ca*Y-Sc*Sa*(Z+d-l);
			J(4,2)=Sc*Sa*Y+Sc*Ca*(Z+d-l);
			J(5,2)=Cc*Sa*Y+Cc*Ca*(Z+d-l);
			J(6,2)=Ca*Y-Sa*(Z+d-l);
			//calcul de l'inverse generalise a gauche (matrice verticale)
			//inverseJ=((J.Transposed()*J).Inverse())*J.Transposed();
			math_SVD Algo(J);
			Algo.PseudoInverse(inverseJ, 1e-12);

			
			v(1)=delta_o.X();	v(2)=delta_o.Y();	v(3)=delta_o.Z();	
			v(4)=delta_p.X();	v(5)=delta_p.Y();	v(6)=delta_p.Z();
	
			deltatheta=inverseJ*v;
			theta+=deltatheta;
		}
		while (deltatheta.Norm()>precision && nb_iter<max_iter);

		A_Coord[i] = new double[NB_JOINT];
		A_Coord[i][0]=theta(3);	
		A_Coord[i][1]=theta(4);	
		A_Coord[i][2]=theta(5);	
		A_Coord[i][3]=theta(2);	
		A_Coord[i][4]=theta(1);	

		min_nb_iter=MIN(min_nb_iter, nb_iter);
		max_nb_iter=MAX(max_nb_iter, nb_iter);
	}
}

void MatsuuraPath::RMROPT()
{
	CWaitCursor c;
	int i,nb_iter;
	int length=ArrayOfPoint.GetSize();
	double X, Y, Z, A, C, Ca, Sa, Cc, Sc;
	gp_XYZ pi, pf, of;
	gp_XYZ delta_p;
	gp_Mat R;
	math_Vector theta(1,5,0);
	math_Vector deltatheta(1,5,0);
	math_Matrix J(1,3,1,5,0);	//J matrice 6*5 initialisee a 0
	math_Matrix inverseJ(1,5,1,3,0);
	math_Vector v(1,3,0);
	math_Vector h(1,5,0);
	math_Vector h2(1,5,0);
	math_Vector theta_moy(1,5,0);
	math_Matrix I5(1,5,1,5,0);
	I5(1,1)=I5(2,2)=I5(3,3)=I5(4,4)=I5(5,5)=1;
	math_Matrix W5(1,5,1,5,0);
	double w0;

	Machine->ComputeW0();
	w0=Machine->w0;
	if (CurCriteria==GLOBAL_CRITERIA)
	{
		Machine->ComputeWeight();

		for (i=1 ; i<=NB_JOINT ; i++)
			W5(i,i)=Machine->Weight[i-1];

		theta_moy(1)=Machine->GetCmoy();
		theta_moy(2)=Machine->GetAmoy();
		theta_moy(3)=Machine->GetXmoy();
		theta_moy(4)=Machine->GetYmoy();
		theta_moy(5)=Machine->GetZmoy();
	}
	else
	{
		gp_Pnt p_moy(0,0,0);
		gp_XYZ o(0,0,0);
		gp_Dir o_moy;
		double Xmax, Ymax, Zmax, Amax, Cmax;

		for (i=0 ; i<ArrayOfPoint.GetSize() ; i++)
		{
			p_moy.SetXYZ(p_moy.Coord()+ArrayOfPoint[i].Coord());
			o+=ArrayOfNormal[i].XYZ();
		}
		p_moy.SetXYZ(p_moy.XYZ()/ArrayOfPoint.GetSize());
		o_moy=gp_Dir(o);
		MGI(p_moy, o_moy, X, Y, Z, A, C);
		Xmax=MAX(ABS(X-Machine->Xmin), ABS(X-Machine->Xmax)); 
		Ymax=MAX(ABS(Y-Machine->Ymin), ABS(Y-Machine->Ymax)); 
		Zmax=MAX(ABS(Z-Machine->Zmin), ABS(Z-Machine->Zmax)); 
		Amax=MAX(ABS(A-Machine->Amin), ABS(A-Machine->Amax)); 
		Cmax=MAX(ABS(C-Machine->Cmin), ABS(C-Machine->Cmax)); 

		W5(1,1)=(1-Machine->WFo)/(NB_JOINT*Cmax*Cmax);
		W5(2,2)=(1-Machine->WFo)/(NB_JOINT*Amax*Amax);

		W5(3,3)=(1-Machine->WFo)/(NB_JOINT*Xmax*Xmax);
		W5(4,4)=(1-Machine->WFo)/(NB_JOINT*Ymax*Ymax);
		W5(5,5)=(1-Machine->WFo)/(NB_JOINT*Zmax*Zmax);
		theta_moy(1)=C;
		theta_moy(2)=A;
		theta_moy(3)=X;
		theta_moy(4)=Y;
		theta_moy(5)=Z;
	}

	A_Coord = new double*[length];

	for (i=0 ; i<length ; i++)
	{
		nb_iter=0;
		nb_iter=0;
		pf=ArrayOfPoint[i].Coord();
		of=ArrayOfNormal[i].XYZ();
		do
		{
			nb_iter++;
			//calcul de pi
			X=theta(3);
			Y=theta(4);
			Z=theta(5);
			A=theta(2);
			C=theta(1);
		
			MGD(X, Y, Z, A, C, pi, R);
			delta_p=pf-pi;

			Ca=cos(A*PI/180);
			Sa=sin(A*PI/180);
			Cc=cos(C*PI/180);
			Sc=sin(C*PI/180);

			//determination de J
			J(1,1)=Sc*X-Cc*Ca*Y+Cc*Sa*(Z+d-l);
			J(2,1)=Cc*X+Sc*Ca*Y-Sc*Sa*(Z+d-l);
			J(1,2)=Sc*Sa*Y+Sc*Ca*(Z+d-l);
			J(2,2)=Cc*Sa*Y+Cc*Ca*(Z+d-l);
			J(3,2)=Ca*Y-Sa*(Z+d-l);
			J(1,3)=-Cc;
			J(2,3)=Sc;
			J(1,4)=-Ca*Sc;
			J(2,4)=-Ca*Cc;
			J(3,4)=Sa;
			J(1,5)=Sa*Sc;
			J(2,5)=Sa*Cc;
			J(3,5)=Ca;
			//calcul de l'inverse generalise a droite (matrice horizontale)
			//inverseJ=J.Transposed()*((J*J.Transposed()).Inverse());
			math_SVD Algo(J);
			Algo.PseudoInverse(inverseJ);
	
			v(1)=delta_p.X();	v(2)=delta_p.Y();	v(3)=delta_p.Z();
			h2(1)=Sa*Cc*of.X()-Sa*Sc*of.Y();
			h2(2)=Ca*Sc*of.X()+Ca*Cc*of.Y()-Sa*of.Z();
			h=W5*(theta-theta_moy)-w0*h2;

			deltatheta=inverseJ*v-(I5-inverseJ*J)*h;
			theta+=deltatheta;
		}
		while (deltatheta.Norm()>precision && nb_iter<max_iter);
		A_Coord[i] = new double[NB_JOINT];
		A_Coord[i][0]=theta(3);	
		A_Coord[i][1]=theta(4);	
		A_Coord[i][2]=theta(5);	
		A_Coord[i][3]=theta(2);	
		A_Coord[i][4]=theta(1);	

		min_nb_iter=MIN(min_nb_iter, nb_iter);
		max_nb_iter=MAX(max_nb_iter, nb_iter);
	}
}


//A et C en degres
void MatsuuraPath::MGD(double X, double Y, double Z, double A, double C, gp_XYZ &p, gp_Mat &R)
{
	p.SetCoord(0,0,0);
	R.SetRows(p, p, p);

	double Ca, Sa, Cc, Sc;

	Ca=cos(A*PI/180);
	Sa=sin(A*PI/180);
	Cc=cos(C*PI/180);
	Sc=sin(C*PI/180);

	p.SetX(-X*Cc-Y*Ca*Sc+(Z+d-l)*Sa*Sc);
	p.SetY(X*Sc-Y*Ca*Cc+(Z+d-l)*Sa*Cc);
	p.SetZ(Y*Sa+(Z+d-l)*Ca-d);

	FillOrientation(A, C, R);
}

bool MatsuuraPath::MGI(gp_Pnt p, gp_Dir o, double &X, double &Y, double &Z, double &A, double &C)
{
	double Ca, Sa, Cc, Sc, x, y, z;

	if (!FindAngle(o, A, C))
		return false;

	x=p.X();	y=p.Y();	z=p.Z();
	Ca=cos(A*PI/180);
	Sa=sin(A*PI/180);
	Cc=cos(C*PI/180);
	Sc=sin(C*PI/180);

	X=-Cc*x+Sc*y;
	Y=-Ca*Sc*x-Ca*Cc*y+Sa*(z+d);
	Z=Sa*Sc*x+Sa*Cc*y+Ca*(z+d)-(d-l);

	return true;
}

void MatsuuraPath::MGI()
{
	CWaitCursor c;
	double X, Y, Z, A, C;
	int i,length=ArrayOfPoint.GetSize();

	A_Coord = new double*[length];

	for (i=0 ; i<length ; i++)
	{
		MGI(ArrayOfPoint[i], ArrayOfNormal[i], X, Y, Z, A, C);
		
		A_Coord[i] = new double[NB_JOINT];
		A_Coord[i][0]=X;	
		A_Coord[i][1]=Y;	
		A_Coord[i][2]=Z;	
		A_Coord[i][3]=A;	
		A_Coord[i][4]=C;	
	}
}

void MatsuuraPath::ComputePath()
{
	if (CurMethod==RMR_METHOD)
		RMR();
	else if (CurMethod==RMROPT_METHOD)
		RMROPT();
	else if (CurMethod==MGI_METHOD)
		MGI();
}