// MatsuuraPath.h: interface for the MatsuuraPath class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_MATSUURAPATH_H__0769342D_C5CE_4DDC_908E_D695B818F677__INCLUDED_)
#define AFX_MATSUURAPATH_H__0769342D_C5CE_4DDC_908E_D695B818F677__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "MatsuuraTool.h"
#include "MatsuuraMachine.h"

//macros
#define ABS(a) ( a<0 ? -a : a )
#define MIN(a,b) ( a<b ? a : b )
#define MAX(a,b) ( a>b ? a : b )

//enumeration
enum CurMethod_
{
	MGI_METHOD,
	RMR_METHOD,
	RMROPT_METHOD
};

enum CurCriteria_
{
	GLOBAL_CRITERIA,
	LOCAL_CRITERIA
};


class MatsuuraPath  
{
public:
	MatsuuraPath();
	virtual ~MatsuuraPath();

public:
	MatsuuraMachine *Machine;
	double **A_Coord;
	CArray<gp_Pnt, gp_Pnt&> ArrayOfPoint;
	CArray<gp_Dir, gp_Dir&> ArrayOfNormal;
	CArray<gp_Mat, gp_Mat&> ArrayOfMat;
	TopTools_SequenceOfShape ArrayOfEdge;
	gp_Pnt StartPnt;
	gp_Dir StartDir;
	gp_Pnt EndPnt;
	gp_Dir EndDir;

protected:
	TopoDS_Face F;
	BRepAdaptor_Surface FAdaptor;
	CArray<BRepAdaptor_Curve, BRepAdaptor_Curve&> ArrayOfCurve;
	CArray<BRepAdaptor_Curve2d, BRepAdaptor_Curve2d&> ArrayOfCurve2d;
	int discret;
	double precision;
	int max_iter;
	int min_nb_iter;
	int max_nb_iter;
	double l,d;
	CurMethod_ CurMethod;
	CurCriteria_ CurCriteria;

public:
	//encapsulation
	void SetMethod(CurMethod_ M) {CurMethod=M;};
	CurMethod_ GetMethod() {return CurMethod;};

	void SetCriteria(CurCriteria_ M) {CurCriteria=M;};
	CurCriteria_ GetCriteria() {return CurCriteria;};

	int GetMinNbIteration() {return min_nb_iter;};
	int GetMaxNbIteration() {return max_nb_iter;};

	void SetPrecision(double val) {precision=val;};
	double GetPrecision() {return precision;};
	
	void SetDiscretisation(int val) {discret=val;};
	int GetDiscretisation() {return discret;};
	
	void SetMaxIter(int val) {max_iter=val;};
	int GetMaxIter() {return max_iter;};

	void SetMachine(MatsuuraMachine *M) {Machine=M;d=Machine->d;l=Machine->Tool->l;};
	MatsuuraMachine* GetMachine() {return Machine;};

	void SetFace(TopoDS_Face &aFace) {F=aFace; FreeA_Coord();};
	TopoDS_Shape& GetFace() {return F;};
	
	int GetNbEdge() {return ArrayOfEdge.Length();};
	void AddEdge(TopoDS_Edge &E) {ArrayOfEdge.Append(E);};
	void SetEdge(int index, TopoDS_Edge &E);
	TopoDS_Edge GetEdge(int index);
	
	void SetCurve(int index, BRepAdaptor_Curve &C);
	BRepAdaptor_Curve& GetCurve(int index);
	
	void SetCurve2d(int index, BRepAdaptor_Curve2d &C2d);
	BRepAdaptor_Curve2d& GetCurve2d(int index);

	int GetNbPoint() {return ArrayOfPoint.GetSize();};
	void SetPoint(int index, gp_Pnt &P);
	gp_Pnt& GetPoint(int index);

	void SetNormal(int index, gp_Dir &N);
	gp_Dir& GetNormal(int index);

	//fonctions
	void Init();
	void InitPath();
	void FillOrientation(double A, double C, gp_Mat &R);
	void FillOrientation(gp_Dir &o, gp_Mat &R, double oldA=0, double oldC=0);
	void FreeA_Coord();
	void ResetArray();
	void ComputeEdge();
	void ComputePointAndNormal();
	void ComputePointAndNormal(int i, double u, gp_Pnt &P, gp_Dir &N);
	void ComputePath();
	void StartEndCycle(gp_Pnt &pi, gp_Dir &oi, gp_Pnt &pf, gp_Dir &of);
	void RMR();
	void RMROPT();
	void MGD(double X, double Y, double Z, double A, double C, gp_XYZ &p, gp_Mat &R);
	bool FindAngle(gp_Dir &o, double &A, double &C, double oldA=0, double oldC=0);
	bool MGI(gp_Pnt p, gp_Dir o, double &X, double &Y, double &Z, double &A, double &C);
	void MGI();
};

#endif // !defined(AFX_MATSUURAPATH_H__0769342D_C5CE_4DDC_908E_D695B818F677__INCLUDED_)
